----------------------------------------------------------------------------------
-- Engineer:       Mike Field <hamster@snap.net.nz>
-- Module Name:    ColourTest - Behavioral 
-- Description:    Generates an 720x480 showing all colours
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity hdmi is
	generic (
		hRez       : natural := 720;	
		hStartSync : natural := 736;
		hEndSync   : natural := 808;
		hMaxCount  : natural := 858;
		hsyncActive : std_logic := '0';

		vRez       : natural := 480;
		vStartSync : natural := 489;
		vEndSync   : natural := 495;
		vMaxCount  : natural := 525;
		vsyncActive : std_logic := '0'
	);

	Port ( 
		clk25	: in  STD_LOGIC;
	
		--hdmi outputs
		hdmi_red	: out STD_LOGIC_VECTOR (7 downto 0);
		hdmi_green	: out STD_LOGIC_VECTOR (7 downto 0);
		hdmi_blue	: out STD_LOGIC_VECTOR (7 downto 0);
		hdmi_hsync	: out STD_LOGIC;
		hdmi_vsync	: out STD_LOGIC;
		hdmi_de		: out STD_LOGIC;
		hdmi_clk	: out STD_LOGIC;
		clk_en	: inout BOOLEAN;
		
		HBlankStart : out STD_LOGIC;
		HBlankEnd : out STD_LOGIC;

		--hdmi control signals
		hcount		: out STD_LOGIC_VECTOR(11 downto 0);
		vcount		: out STD_LOGIC_VECTOR(11 downto 0)
	);
end hdmi;

architecture Behavioral of hdmi is

	type reg is record
		hcounter : std_logic_vector(11 downto 0);
		vcounter : std_logic_vector(11 downto 0);

		red      : std_logic_vector(7 downto 0);
		green    : std_logic_vector(7 downto 0);
		blue     : std_logic_vector(7 downto 0);

		hsync    	: std_logic;
		vsync    	: std_logic;
		blank    	: std_logic;
	end record;

	signal r : reg := ((others=>'0'), (others=>'0'), (others=>'0'), (others=>'0'), (others=>'0'), '0', '0', '0');
	signal n : reg	:= ((others=>'0'), (others=>'0'),(others=>'0'), (others=>'0'), (others=>'0'), '0', '0', '0');

	signal off : std_logic_vector (7 downto 0) := (others => '0');

begin

	-- Assign the outputs
	hdmi_hsync	<= r.hSync;
	hdmi_vsync	<= r.vSync;
	hdmi_red	<= r.red;
	hdmi_green	<= r.green;
	hdmi_blue	<= r.blue;
	hdmi_de		<= r.blank;
	hdmi_clk	<= clk25;
	
	hcount 		<= r.hcounter;
	vcount		<= r.vcounter;
	
	process(r, n)
	begin
			n <= r;
			if(clk_en) then
			
				n.hSync <= not hSyncActive;      
				n.vSync <= not vSyncActive;      

				-- Count the lines and rows      
				if r.hCounter = hMaxCount-1 then
					n.hCounter <= (others => '0');
					if r.vCounter = vMaxCount-1 then
						n.vCounter <= (others => '0');
					else
						n.vCounter <= r.vCounter + 1;
					end if;
				else
					n.hCounter <= r.hCounter + 1;
				end if;
				
			
				if r.hCounter < hRez and r.vCounter < vRez then
					n.red   <= n.hCounter(5 downto 0) & n.hCounter(5 downto 4);
					n.green <= n.hCounter(7 downto 0);
					n.blue  <= n.vCounter(7 downto 0) + off;

	--				n.red   <= (others => '1');
	--				n.green <= (others => '0');
	--				n.blue  <= (others => '1');
					n.blank <= '0';
				else
					n.red   <= (others => '0');
					n.green <= (others => '0');
					n.blue  <= (others => '0');
					n.blank <= '1';
				end if;
				
				-- Are we in the hSync pulse?
				if r.hCounter >= hStartSync and r.hCounter < hEndSync then
					n.hSync <= hSyncActive;
				end if;

				-- Are we in the vSync pulse?
				if r.vCounter >= vStartSync and r.vCounter < vEndSync then
					n.vSync <= vSyncActive; 			
				end if;
			end if;
	end process;
	
	process (clk25, n)
	begin
		if rising_edge(clk25) then
			r <= n;
			clk_en <= not clk_en;
			
			if r.vCounter = vStartSync and r.hCounter = 0 then
				off <= off + 1;
			end if;
			

			if(clk_en) then
				if(r.hCounter = hrez-2) then
					HBlankStart <= '1';
				else
					HBlankStart <= '0';
				end if;	
				if(r.hCounter = hMaxCount-12) then
					HBlankEnd <= '1';
				else
					HBlankEnd <= '0';
				end if;
			end if;
		end if;
	end process;
	
end Behavioral;
